////////////////////////////////////// c++ /////////////////////////////////////
//
//  Module           : parser.cpp
//  Description      :
//  Author           : Dino Ahr
//  Email            : Dino.Ahr@informatik.uni-heidelberg.de
//  Copyright        : University of Heidelberg
//  Created on       : Mon Oct 29 08:55:19 2001
//  Last modified by : ahr
//  Last modified on : Tue Oct 30 11:59:42 2001
//  Update count     : 34
//
////////////////////////////////////////////////////////////////////////////////
//
//  Date	Name		Changes/Extensions
//  ----	----		------------------
//
////////////////////////////////////////////////////////////////////////////////


#include "parser.hh"
#include <iostream>
#include <stdio.h>

//
// C o n s t r u c t o r
//
Parser::Parser() :
  m_lookaheadToken(NONE)
{
  // Create symbol table
  m_symbolTable = new SymbolTable();

  // Insert key words
  m_symbolTable->insert("div", DIV);
  m_symbolTable->insert("mod", MOD);

  // Create scanner
  m_scanner = new Scanner(m_symbolTable);
}	

//
// D e s t r u c t o r
//
Parser::~Parser(){

  // Delete scanner
  delete m_scanner;

  // Delete symbol table
  delete m_symbolTable;
}	

//
// p a r s e
//
void Parser::parse()            
{
  m_lookaheadToken = m_scanner->getNextToken();
  while (m_lookaheadToken != DONE) {
    expr(); 
    match (SEM);
  }
}

//
// e x p r
//
void Parser::expr()
{
  term();
  while(1) {
    switch(m_lookaheadToken) {
    case PLUS: 
    case MINUS:
      {
	Token t = m_lookaheadToken;
	match(m_lookaheadToken); 
	term(); 
	emit(t);
	break;
      }
    default:
      return;
    } // switch
  } // while

} // expr

//
// t e r m
//
void Parser::term()
{
  factor();
  while(1) {
    switch(m_lookaheadToken) {
    case MULT: 
    case SLASH:
    case DIV: 
    case MOD:
      {
	Token t = m_lookaheadToken;
	match(m_lookaheadToken); 
	factor(); 
	emit(t);
	break;
      }
    default:
      return;
    } // switch
  } // while

} // term

//
// f a c t o r
//
void Parser::factor()
{
  switch(m_lookaheadToken) {
  case OPEN:
    match (OPEN); 
    expr(); 
    match (CLOSE); 
    break;
  case NUM:
    emit(NUM, m_scanner->getTokenValue());
    match(NUM); 
    break;
  case ID:
    emit(ID, m_scanner->getTokenValue());
    match(ID); 
    break;
  default:
    m_scanner->error("Syntax error");
  }
} // factor

//
// m a t c h 
//
void Parser::match(Token t)
{
  // Line break if semicolon has been reached
  if (t == SEM){
    cout << endl;
  }

  if (m_lookaheadToken == t) {
    m_lookaheadToken = m_scanner->getNextToken();
  }
  else {
    m_scanner->error("Syntax error");
  }

} // match

//
// e m i t
//
void Parser::emit(Token t, int tokenValue)      
{
  switch(t) {
  case PLUS: 
    printf("+ "); 
    break;
  case MINUS: 
    printf("- "); 
    break;
  case MULT: 
    printf("* "); 
    break;
  case SLASH:
    printf("/ "); 
    break;
  case DIV:
    printf("DIV "); 
    break;
  case MOD:
    printf("MOD "); 
    break;
  case NUM:
    printf("%d ", tokenValue); 
    break;
  case ID:
    printf("%s ", (m_symbolTable->getEntry(tokenValue))->m_lexPtr); 
    break;
  default:
    cerr << "Invalid token" << endl;
    exit(1);
    break;
  }

} // emit



