////////////////////////////////////// c++ /////////////////////////////////////
//
//  Module           : scanner.cpp
//  Description      :
//  Author           : Dino Ahr
//  Email            : Dino.Ahr@informatik.uni-heidelberg.de
//  Copyright        : University of Heidelberg
//  Created on       : Mon Oct 29 08:55:19 2001
//  Last modified by : ahr
//  Last modified on : Tue Oct 30 11:58:06 2001
//  Update count     : 26
//
////////////////////////////////////////////////////////////////////////////////
//
//  Date	Name		Changes/Extensions
//  ----	----		------------------
//
////////////////////////////////////////////////////////////////////////////////


#include "scanner.hh"
#include "symbolTable.hh"
#include <iostream>
#include <cassert>
#include <stdio.h>
#include <ctype.h>

//
// C o n s t r u c t o r
//
Scanner::Scanner(SymbolTable *symbolTable) :
  m_symbolTable(symbolTable),
  m_lineNumber(1),
  m_tokenValue(-1)
{
  assert(m_symbolTable != 0);
}	

//
// g e t N e x t T o k e n
//
Token Scanner::getNextToken()
{
  while(1) {

    int t = getchar();

    // Eliminate blanks
    if (t == ' ' || t == '\t') {
      ;
    }

    // New line
    else if (t=='\n') {
      ++m_lineNumber;
    }

    // Digit
    else if (isdigit(t)) {
      ungetc(t,stdin);
      scanf("%d", &m_tokenValue);
      return NUM;
    }

    // Letter
    else if (isalpha(t)) {     

      char lexbuf[128];
      int p, b = 0;

      // Copy identifier to lexbuf
      while (isalnum(t)) {  /* t is alphanumeric */
        lexbuf[b] = t;
        t = getchar();
        b++;
        if (b >= 128) {
          cerr << "Compiler error!" << endl;
	  exit(1);
        }
      }
      lexbuf[b]= '\0';
      if (t != EOF) { 
        ungetc(t,stdin);
      }

      // Put into symbol table if does not exist
      p = m_symbolTable->lookup(lexbuf);
      if (p == 0) {
        p = m_symbolTable->insert(lexbuf, ID);
      }
      m_tokenValue = p;
      return (m_symbolTable->getEntry(p))->m_token;
    }

    // End of file
    else if (t == EOF) {
      return DONE;
    }

    // End of statement
    else if (t == ';') {
      return SEM;
    }

    // Open brace
    else if (t == '(') {
      return OPEN;
    }

    // Close brace
    else if (t == ')') {
      return CLOSE;
    }

    // Slash
    else if (t == '/') {
      return SLASH;
    }

    // Plus
    else if (t == '+') {
      return PLUS;
    }

    // Minus
    else if (t == '-') {
      return MINUS;
    }

    // Mult
    else if (t == '*') {
      return MULT;
    }

    // No valid token
    else {
      return NONE;
    }

  } // while (1)

} // getNextToken

//
// e r r o r
//
void Scanner::error(char * message) const
{
  cout << "ERROR: line:" << m_lineNumber << " " << message << endl;
  cout << "Interrupted.\n" << endl;
  exit(1);
} // error
