////////////////////////////////////// c++ ////////////////////////////////////
//
//  Module           : scanner.hh
//  Description      :
//  Author           : Dino Ahr
//  Email            : Dino.Ahr@informatik.uni-heidelberg.de
//  Copyright        : University of Heidelberg
//  Created on       : Mon Oct 29 08:53:30 2001
//  Last modified by : ahr
//  Last modified on : Tue Oct 30 11:56:27 2001
//  Update count     : 15
//
///////////////////////////////////////////////////////////////////////////////
//
//  Date	Name		Changes/Extensions
//  ----	----		------------------
//
///////////////////////////////////////////////////////////////////////////////


#ifndef SCANNER_HH
#define SCANNER_HH

/**
 * This enum type contains the token types detected by the scanner and processed
 * by the parser.  
 */
enum Token {
  NONE,      ///< Invalid token
  NUM,       ///< An integer
  DIV,       ///< "div"
  SLASH,     ///< "/"
  MOD,       ///< "mod"
  ID,        ///< Identifier
  SEM,       ///< ";"
  PLUS,      ///< "+"
  MINUS,     ///< "-"
  MULT,      ///< "*"
  OPEN,      ///< "("
  CLOSE,     ///< ")"
  DONE       ///< End of file reached
}; // enum Token 

// Forward
class SymbolTable;

/**
 * This class represents the scanner.
 */
class Scanner {

private:

  // -----------------------------------------------------
  // -------- V a r i a b l e s --------------------------
  // -----------------------------------------------------

  /**
   * Pointer to the symbol table.
   */
  SymbolTable *m_symbolTable;

  /**
   * The current line number of the input stream.
   */
  int m_lineNumber;

  /**
   * 
   */
  int m_tokenValue;

public:

  // -----------------------------------------------------
  // ------- M e t h o d s   ( p u b l i c ) -------------
  // -----------------------------------------------------

  /**
   * Constructor.
   */
  Scanner(SymbolTable *symbolTable);

  /**
   * Returns the next token from the input stream. If the token is NUM the value
   * of the number can be ontained by the function getTokenValue(). If the token
   * is ID the function getTokenValue() returns the index of symbol table entry
   * of the identifier. This index can be used to obtain the identifiers name
   * from the symbol table.
   *
   * @return The next token from the input stream.  */
  Token getNextToken();

  /**
   * Returns the current token value.
   *
   * @return The current token value.
   */
  int getTokenValue() const {
    return m_tokenValue;
  }

  /**
   * Prints an error message and terminates the program.
   */
  void error(char * message) const;

}; // class Scanner


#endif
