////////////////////////////////////// c++ /////////////////////////////////////
//
//  Module           : symbolTable.cpp
//  Description      :
//  Author           : Dino Ahr
//  Email            : Dino.Ahr@informatik.uni-heidelberg.de
//  Copyright        : University of Heidelberg
//  Created on       : Mon Oct 29 08:55:19 2001
//  Last modified by : ahr
//  Last modified on : Mon Oct 29 13:34:31 2001
//  Update count     : 21
//
////////////////////////////////////////////////////////////////////////////////
//
//  Date	Name		Changes/Extensions
//  ----	----		------------------
//
////////////////////////////////////////////////////////////////////////////////


#include "symbolTable.hh"
#include <iostream>
#include <cassert>

//
// C o n s t r u c t o r
// 
SymbolTable::SymbolTable() :
  m_lastTableEntry(0),
  m_lastLexArrayPosition(-1)
{
  // Initialize symboltable
  for (int i = 0; i < s_maxTableSize; ++i){
    m_symbolTable[i] = 0;
  }
}	

//
// D e s t r u c t o r
// 
SymbolTable::~SymbolTable()
{
  // Delete entries of the symbol table
  for (int i = 1; i <= m_lastTableEntry; ++i) {
    delete m_symbolTable[i];
  }
}

//
// l o o k u p 
//
int SymbolTable::lookup(char s[]) const
{
  // Traverse symbol table
  for (int p = m_lastTableEntry; p > 0; --p) {
    if (strcmp((m_symbolTable[p])->m_lexPtr, s) == 0) {
      return p;
    }
  }

  // Symbol not found
  return 0;

} // lookup

//
// i n s e r t
// 
int SymbolTable::insert(char s[], Token token) 
{

  int len = strlen(s);

  if (m_lastTableEntry + 1 >= s_maxTableSize) {
    cerr << "Symbol table overflow!" << endl;
    exit(1);
  }

  if (m_lastLexArrayPosition + len + 1 >= s_maxLengthLexArray) {
    cerr << "Lexem array overflow!" << endl;
    exit(1);
  }

  // Create new table entry
  m_lastTableEntry++;
  m_symbolTable[m_lastTableEntry] = new TableEntry();
  m_symbolTable[m_lastTableEntry]->m_token = token;
  m_symbolTable[m_lastTableEntry]->m_lexPtr = &m_lexArray[m_lastLexArrayPosition + 1];

  // Store lexem string
  m_lastLexArrayPosition = m_lastLexArrayPosition + len + 1;
  strcpy(m_symbolTable[m_lastTableEntry]->m_lexPtr, s);

  return m_lastTableEntry;

} // insert

//
// g e t E n t r y
//
TableEntry *SymbolTable::getEntry(int i)
{

  assert( (i > 0) && (i <= m_lastTableEntry) );
  return m_symbolTable[i];

} // getEntry
