////////////////////////////////////// c++ ////////////////////////////////////
//
//  Module           : symbolTable.hh
//  Description      :
//  Author           : Dino Ahr
//  Email            : Dino.Ahr@informatik.uni-heidelberg.de
//  Copyright        : University of Heidelberg
//  Created on       : Mon Oct 29 08:37:15 2001
//  Last modified by : ahr
//  Last modified on : Mon Oct 29 12:56:36 2001
//  Update count     : 27
//
///////////////////////////////////////////////////////////////////////////////
//
//  Date	Name		Changes/Extensions
//  ----	----		------------------
//
///////////////////////////////////////////////////////////////////////////////


#ifndef SYMBOL_TABLE_HH
#define SYMBOL_TABLE_HH

#include "parser.hh"
#include "scanner.hh"

/**
 * This struct represents one entry of the symbol table.
 */
struct TableEntry {
  
  /**
   * Pointer to the lexem.
   */
  char *m_lexPtr;

  /**
   * The token of the entry.
   */
  Token m_token;

}; // struct TableEntry

/**
 * This class maintains the storage of keywords and lexems.
 */
class SymbolTable {

private:

  // -----------------------------------------------------
  // -------- V a r i a b l e s --------------------------
  // -----------------------------------------------------

  /**
   * Maximal size of the table.
   */
  static const int s_maxTableSize = 100;

  /**
   * Array which stores the single entries.
   */
  TableEntry *m_symbolTable[s_maxTableSize];

  /**
   * Last used table entry.
   */
  int m_lastTableEntry;

  /**
   * Maximal length of the lexemes array.
   */
  static const int s_maxLengthLexArray = 999;

  /**
   * Array which stores the names of the lexems.
   */
  char m_lexArray[s_maxLengthLexArray];

  /**
   * Index of the last used position of a lexem.
   */
  int m_lastLexArrayPosition;


public:

  // -----------------------------------------------------
  // ------- M e t h o d s   ( p u b l i c ) -------------
  // -----------------------------------------------------
  
  /**
   * Constructor.
   */
  SymbolTable();

  /**
   * Destructor.
   */
  ~SymbolTable();
  
  /** 
   * Check whether symbol \a s is in table.
   *
   * @param s The symbol to be checked for being contained in the table.
   *
   * @return The index of the entry if symbol has been found, 0 otherwise.  
   */
  int lookup(char s[]) const;

  /**
   * Inserts a token-string pair into the symbol table.
   *
   * @param s The string.
   *
   * @param token The token.
   * 
   * @return The index of the newly created entry.
   */
  int insert(char s[], Token token);

  /**
   * Returns the table entry with index \a i.
   */
  TableEntry *getEntry(int i);

}; // class SymbolTable


#endif
